from typing import TypedDict


class BasketballCourtSettings(TypedDict):
    backgroundColor: str
    courtColor: str
    paintColor: str
    threePointsColor: str
    linesColor: str
    sidePaintColor: str
    linesWidth: int
